/*
	DWaveCmd.h - selection menu for loaded wave files

	Will Eighteen, 2001: will@opaline.demon.co.uk
	Dynamic menu: code Xi Eighteen, 2001, implementation Will Eighteen, 2001
*/

#ifndef DWAVEMENUCMD_H
#define DWAVEMENUCMD_H

#define MENUSTORAGESPACE 16384
#define BUFSIZE 1024

#include "NoUndoCmd.h"
#include <Xm/Xm.h>


typedef void ( *DWaveCmdCallback ) ( void *, unsigned int, char * );

class DWaveCmd;

struct waveWidgetSet { int n; DWaveCmd *waveMenu; Widget *toggle; };


class DWaveCmd : public NoUndoCmd
{
	public:
		// name, parent widget, menu popup label, 
		// menu activate button label, menu entry list title,
		// menu entries
		DWaveCmd ( const char *, int active, Boolean quitOnSelect,
					DWaveCmdCallback, void *, char *, ... );
					
		// instead of fixed string menu entries, will also accept
		// no. of items in menu, size of char array storing entries,
		// pointer to base address of char array
		DWaveCmd ( const char *, int active, Boolean quitOnSelect,
					 DWaveCmdCallback, void *, char *, int, int, char * );
		 
		~DWaveCmd();
		virtual const char *const className() { return ( "DWaveCmd" ); }
		
	protected:
		void doCommand();		// called by base class
		DWaveCmdCallback _callback;	// function to be called
		void *_clientData;		// data provided by caller

	private:
		static void menuEntrySelectCallback( Widget, XtPointer, XtPointer );
		static void opModeCallback ( Widget, XtPointer, XtPointer );

		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer,XtPointer  );

		void clientMessage ( XmAnyCallbackStruct * );
 		void menuEntrySelect ( unsigned int );
		void setSelectedItem ( unsigned int n ) { _selectedItemNumber = n; }
		void quit();		
		void opMode ( waveWidgetSet *, unsigned int );
		char * stripName ( char * );
		char * itemToLabel ( unsigned int );

		Boolean _fixedString;
		Boolean _quitOnSelect;	// if menu is to be popped down on selection
		Boolean _popupRealised;
		Widget _popup, _menuLabel, _menu, _rc, _entry, _toggle;
		char _name[128];
		char _applicationName[128];
		char *_pItemArray;
		char _menuItemList[MENUSTORAGESPACE];
		char *_menuLabelText;
		char _fileName[BUFSIZE];
		char _label[BUFSIZE];
		unsigned int _numMenuItems;
		unsigned int _charArraySize;
		unsigned int _selectedItemNumber;
		waveWidgetSet ** arrayOfPointersToMemBlocks;
};

#endif

/*
	Notes:

	Since we can't use varargs ploy to access the menu item list outside the
	constructor, in doCommand, we must store the data internally. Thus the menu
	item capability is restricted to this storage limit, 'MENUSTORAGESPACE'.

	Usage: see Command/DMenu

	Note that each menu must reserve the last menu item for closing the popup
	menu.

	Tnis is a modified version of DMenuCmd producing a popup of wave names with
	associated toggle button. The state of the toggle sets the opMode for the
	wave in the global wavePak structure for the wave, (*wavePak)[n].opMode.
	If this value is zero, then the toggle button appears unset (raised); if 1,
	then the toggle appears set (depressed).

	objects may reference the wavePak object's opMode member to determine whether
	an action is to be performed on the entire wave data or just a selection
	indicated by the values of the wavePak members startSel and endSel. opMode
	value 0 signifies 'all data', opMode 1 signifies 'selection'.

	If the 'selection' state is used, the calling code should also check for
	startSel != endSel, i.e. that a valid range of data is selected.
*/
