/*
	MixwaveUtils.cc

	Will Eighteen, 2001: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "MixwaveUtils.h"
#include <MessageDialogManager.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>


MixwaveUtils::MixwaveUtils()
{
	// empty
}


MixwaveUtils::~MixwaveUtils()
{
	// empty
}


char * MixwaveUtils::fileFromPath ( char *path )
{
	int i = strlen ( path );
	unsigned int j = 0;

	while  ( ( path[i] != '/' ) && ( i > -1 ) )
		i--;
	
	i++;

	while ( i < ( int ) strlen ( path ) )
	{
		if ( j == sizeof ( _fileName ) )
			i = strlen ( path );
		else
		{
			_fileName[j] = path[i];
			i++;
			j++;
		}
	}

	_fileName[j] = 0;
	return ( char * ) &_fileName;
}


char * MixwaveUtils::sampleNumToTimeString ( long sampleNum, long sampleRate, unsigned int lenTimeString, unsigned int fpDigits )
{
	long minutes;
	double fValue = sampleNum;
	char formatString[2*lenTimeString+1];

	fValue /= ( double ) sampleRate;
	minutes = ( long ) ( fValue / 60 );
	fValue -= ( double ) minutes * 60;
	snprintf ( formatString, sizeof ( formatString ),"%ld%s%d%s", minutes, ":%.", fpDigits, "lf" );
	formatString [ sizeof ( formatString ) - 1 ] = 0;
	snprintf ( _timeString, sizeof ( _timeString ), formatString, fValue);
	_timeString [ sizeof ( _timeString ) - 1 ] = 0;
	return ( char * ) &_timeString;
}


long MixwaveUtils::timeStringToSampleNum ( char *timeString, long sampleRate )
{
	long value = 0;
		
	if ( !checkTimeStringFormat ( timeString ) )	// validated format and digits
	{
		long minutes;
		double fSeconds;
		
		sscanf ( timeString, "%ld:%lf", &minutes, &fSeconds );
		value = minutes * 60 * sampleRate;
		value += ( long ) ( fSeconds * sampleRate + 0.5 );
	}

	return  value;
}


int MixwaveUtils::checkTimeStringFormat ( char *text )
{
	int error = 0;
	int length = strlen ( text );
 	char c = 0;

	for ( int count = 0; count < length; count++ )
	{
		c = text [ count ];

		if ( ( c < '0' ) || ( c > '9' ) )
		{
			if ( ( c == ':' ) || ( c == '.' ) )
				error = 0;
			else
			{
				char msg[24];
				sprintf ( msg, "Error: '%c' non-numeric.", c );
				messageDialogManager->post( msg );
				error = 1;
			}
		}
	}

	if ( error )
	{
		// assume invalid, pop up format msg
		char msg[strlen ( text ) + 58];
		snprintf ( msg, sizeof ( msg ), "Invalid format %s\nUse x<:y><.y> format, x = mins, y = secs.",  text );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
	}
	
	return error;
}


int MixwaveUtils::sysCall ( const char *command )
{	// code ripped from GNU 'system' man page
	int pid, status;

	if ( command == 0 )
		return 1;

	pid = fork();

	if ( pid == -1 )
		return -1;

	if ( pid == 0 )
	{
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = ( char * ) command;
		argv[3] = 0;
		execve( "/bin/sh", argv, environ );
		_exit ( 127 );
	}

	do
	{
		if ( waitpid ( pid, &status, 0 ) == -1 )
		{
			if ( errno != EINTR )
				return -1;
		}
		else
			return status;

	} while ( 1 );
}
