; commented_spatial.orc
; expt. to control apparent position of sound source in the plane

; 'csound -W -o <out.wav> <name.orc> <name.sco>'

#define PI # 3.14159265359 #
#define ROOT2 # 1.414213562 #
#define SVEL # 231.68 #

sr = 44100
kr = 4410
ksmps = 10
nchnls = 2

garvbsig init 0; make zero at orch init time

instr 1
ipanfn = p4
ifiltfn = p5
iskiptime = 0.00000 
kemit = p6; angular displacement of sound sources
kampscl = p7
idetune = p8
iphasectl = p9

arvbsig soundin "<name>_reverb.flt", <pitch>, 0.00000 , 6
arangesig soundin "<name>_range.flt", <pitch>, 0.00000 , 6
abwsig soundin "<name>_bwidth.flt", <pitch>, 0.00000 , 6
asweepsig soundin "<name>_sweep.flt", <pitch>, 0.00000 , 6
apansig soundin "<name>_pan.flt", <pitch>, 0.00000 , 6
adplrsig soundin "<name>_dplr.flt", <pitch>, 0.00000 , 6

; we need k-rate equivalents - no downsampling since the pitch value above
; ensures the control files are time-stretched to the correct length 
; (allows use of small control files)

kdplr downsamp adplrsig
ksweep downsamp asweepsig
kbw downsamp abwsig

; panned signal has a delay associated with it due to differing path to ears
ie = 0.075; a constant, approx. half inter-aural distance

; find Cartesian expression of source position
ay = sin (apansig * $PI)
ax = cos (apansig * $PI)

; hence find hypotenuses
apath1 = sqrt(((ax-ie) * (ax-ie)) + (ay * ay))
apath2 = sqrt(((ax+ie) * (ax+ie)) + (ay * ay))

; hence find path difference
adiff = apath1 - apath2

; sign of kdiff governs whether delay should be applied to L or R channel

; find delay between path to ears
adly = adiff / $SVEL; delay time in seconds

; make delay time +ve
adly = sqrt(adly * adly)

; start of sound file processing; load the sample to be used
; block: source -> doppler

; if pitch change, then do phasing here
asiga diskin "<name>.wav", adplrsig, iskiptime
asigb = 0
asigc = 0

if p9 = 0 goto endcascade; turn off 'cascade harmonics'
; if kdplr = 1 we don't have a pitch change, otherwise
; we must simulate 'phasing' effect of receding source

if kdplr = 1 goto endcascade
; slightly detuned inputs to give a phasing effect on pitch change
adetune = (adplrsig * (abs(arangesig))) / idetune
kdetune downsamp adetune
asigb diskin "<name>.wav", kdplr+kdetune, iskiptime
asigc diskin "<name>.wav", kdplr-kdetune, iskiptime
asig = asig * 3/5
endcascade:

; global amplitude control
; block: ampl (fade-with-distance)
asig = (1 / arangesig) * asig

if (ksweep = 0) goto skipfilter

; LF is predominantly responsible for spatial cues, acc. to
; Wightman & Kistler, abstract, 'Dominant Role of Low-Frequency Interaural
; Time Differences in Sound Localization', Acou.Soc.Am V.91
; the relative phase diff would be greater.

; apply a filter whose characteristics are time-variant over the sample
; block: vb bp filt (simulate absorption by transmission medium)
if ifiltfn = 1 goto notch
asig reson asig, ksweep, kbw, 1
goto skipfilter

notch:
; asig areson asig, ksweep, kbw, 1
asig1 tone asig, ksweep-kbw/2
asig2 atone asig, ksweep+kbw/2
asig = asig1+asig2

skipfilter:

; left source diff > 0 < right source diff
; delay runs from zero at centre to kdiff at either extreme
; block: L/R delay

asigd  vdelay asig, adly*1000, 29; convert to msec
; we now output asig, asigd, and reverb separately

; apansig ranges from 0 (L)  to 1 (R)
; note: a linear pan leads to a 1/sqrt(2) power loss at centre

if ipanfn = 1 goto pan1
if ipanfn = 2 goto pan2

; pan 0 - constant power
a1 = apansig * $PI / 2
a2 = cos ( a1 )
a3 = sin ( a1 )
goto endpan

; pan 1
; Michael Gogins suggested another panning method in a post to the
; Csound mailing list - Csound Ezine Aug. 2000 http://www.werewolf.net
; this one has a strange 'ringing' effect
pan1:
a1 = apansig * $PI
a2 = sqrt ( 2 ) / 2 * ( cos ( a1 - $PI / 2 ) - sin ( a1 - $PI / 2 ) )
a3 = sqrt ( 2 ) / 2 * ( cos ( a1 - $PI / 2 ) + sin ( a1 - $PI / 2 ) )
goto endpan

; pan2
; derived from Kim Cascone, 'Ambient-CSound'.This causes phase inversion between
; the L/R channels. Compare with pan1 above, where both channels are in phase.
; This pan has the same effect as pan1, but is easier on the ears, having a
; less reverberant quality.
pan2:
a1 = apansig * $PI
a2 = 1/sqrt(2)*(cos(a1)+sin(a1))
a3 = 1/sqrt(2)*(cos(a1)-sin(a1))

endpan:

; default the channel outputs to direct signal
apath1 = asig
apath2 = asig
; substitute delayed signal in opposite channel to location of source
kdiff downsamp adiff
if kdiff = 0 kgoto endxpos

if kdiff < 0 kgoto objectright
; kdiff is +ve
apath2=asigd
kgoto endxpos
objectright:
; kdiff is -ve
apath1 = asigd
endxpos:

aoutl = apath1 * a2 * kampscl
aoutr = apath2 * a3 * kampscl

; stereo output
outs aoutl, aoutr
; calculate amount of reverb signal to send, don't need a return, is output
; separately to main signal

garvbsig = garvbsig + asig * arvbsig * kampscl
endin


instr 99; global reverb unit - reflections

if (p10 == 0) goto skipreverb

ipanfnl = p4
ipanfnr = p5
idelayl = p6
idelayr = p7
ifiltfnl = p8
ifiltfnr = p9

arvblsig soundin "<name>_rvbl.flt", <pitch> , 6
arvbrsig soundin "<name>_rvbr.flt", <pitch> , 6
apanlsig soundin "<name>_panl.flt", <pitch> , 6
apanrsig soundin "<name>_panr.flt", <pitch> , 6
abwlsig soundin "<name>_bwl.flt", <pitch> , 6
abwrsig soundin "<name>_bwr.flt", <pitch> , 6
asweeplsig soundin "<name>_swpl.flt", <pitch> , 6
asweeprsig soundin "<name>_swpr.flt", <pitch> , 6
aamplsig soundin "<name>_ampl.flt", <pitch> , 6
aamprsig soundin "<name>_ampr.flt", <pitch> , 6

krvbl downsamp arvblsig
krvbr downsamp arvbrsig
kbwl downsamp abwlsig
kbwr downsamp abwrsig
ksweepl downsamp asweeplsig
ksweepr downsamp asweeprsig

; left reflection pan
if ipanfnl = 1 goto pan2l

a1l = apanlsig * $PI / 2
a2l = cos ( a1l )
a3l = sin ( a1l )
goto endpanl

pan2l:
a1l = apanlsig * $PI
a2l = sqrt ( 2 ) / 2 * ( cos ( a1l - $PI / 2 ) - sin ( a1l - $PI / 2 ) )
a3l = sqrt ( 2 ) / 2 * ( cos ( a1l - $PI / 2 ) + sin ( a1l - $PI / 2 ) )

endpanl:

; right reflection pan
if ipanfnr = 1 goto pan2r

a1r = apanrsig * $PI / 2
a2r = cos ( a1r )
a3r = sin ( a1r )
goto endpanr

pan2r:
a1r = apanrsig * $PI
a2r = sqrt ( 2 ) / 2 * ( cos ( a1r - $PI / 2 ) - sin ( a1r - $PI / 2 ) )
a3r = sqrt ( 2 ) / 2 * ( cos ( a1r - $PI / 2 ) + sin ( a1r - $PI / 2 ) )
endpanr:

; start reverb / reflection processing
; left path filter
; block LH filter
asigl = garvbsig

if (kbwl == 0) goto endfiltl
if ifiltfnl = 1 goto notchl
asigl reson garvbsig, ksweepl, kbwl, 1
goto endfiltl

notchl:
; asigl areson garvbsig, asweeplsig, abwlsig, 1
asigl1 tone garvbsig, ksweepl-kbwl/2
asigl2 atone garvbsig, ksweepl+kbwl/2
asigl = asigl1+asigl2

endfiltl:

; block RH filter
; right path filter
asigr = garvbsig

if (kbwr == 0) goto endfiltr
if ifiltfnr = 1 goto notchr
asigr reson garvbsig, ksweepr, kbwr, 1
goto endfiltr

notchr:
; asigr areson garvbsig, asweeprsig, abwrsig, 1
asigr1 tone garvbsig, ksweepr-kbwr/2
asigr2 atone garvbsig, ksweepr+kbwr/2
asigr = asigr1+asigr2

endfiltr:

; block reverb
aoutl reverb asigl, krvbl; left reflection
aoutr reverb asigr, krvbr; right reflection

; need separate delay for each path - can't simulate echo type lengths
; with just a reverb

; block delay
; left path delay
asigl = aoutl; default

if (p6 == 0) goto nodlyl
asigl delay aoutl, idelayl
nodlyl: 

; right path delay
asigr = aoutr

if (p7 == 0) goto nodlyr
asigr delay aoutr, idelayr
nodlyr:

;now make adjustment for phase diff in path to ears
ie = .075; a constant. approx: half inter-aural distance

ay = sin (apanlsig * $PI)
ax = cos (apanlsig * $PI)

apath1 = sqrt(((ax-ie) * (ax-ie)) + (ay * ay))
apath2 = sqrt(((ax+ie) * (ax+ie)) + (ay * ay))

adiffl = apath1 - apath2

adlyl = adiffl / 231.68 ; delay time in seconds
adlyl = sqrt(adlyl * adlyl)

ay = sin (apanrsig * $PI)
ax = cos (apanrsig * $PI)

apath1 = sqrt(((ax-ie) * (ax-ie)) + (ay * ay))
apath2 = sqrt(((ax+ie) * (ax+ie)) + (ay * ay))

adiffr = apath1 - apath2

adlyr = adiffr / $SVEL ; delay time in seconds
adlyr = sqrt(adlyr * adlyr)

asigl = asigl * aamplsig; LH amplitude
asigr = asigr * aamprsig; RH amplitude

asigld  vdelay asigl, adlyl*1000, 29
asigrd vdelay asigr, adlyr*1000, 29

; left reflection adiff > 0 < left source adiff

alpath1 = asigl
alpath2 = asigl

; left hand signal paths
kdiffl downsamp adiffl
if kdiffl = 0 kgoto endxposl

; kdiffl is +ve
if kdiffl < 0 kgoto objectrightl
alpath2 = asigld
kgoto endxposl

objectrightl:
; kdiffl is -ve
alpath1 = asigld
endxposl:

; right reflection kdiff > 0 < right source kdiff

arpath1 = asigr
arpath2 = asigr

; right hand signal paths
kdiffr downsamp adiffr
if kdiffr = 0 kgoto endxposr

; kdiffr is +ve
if kdiffr < 0 kgoto objectrightr
arpath2 = asigrd
kgoto endxposr

objectrightr:
; kdiffr is -ve
arpath1 = asigrd
endxposr:

; left reflection stereo components
outs alpath1 * a2l, alpath2 * a3l
; right reflection stereo components
outs arpath1 * a2r, arpath2 * a3r

garvbsig = 0; clear global reverb

skipreverb:

endin

; end commented_spatial.orc
