/*
 Application.h - app launcher

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall
*/

#ifndef APPLICATION_H
#define APPLICATION_H

#include "Interface.h"
#include "List.h"


/* These are defaults for _all_ apps built with this framework */
static String fallback_resources[] = {
	"*foreground: black",
   	"*background: grey80",
	"*highlightColor: grey90",
	"*marginWidth: 3",
	"*marginHeight: 2",
	NULL,
};

class MainWindow;

class Application : public Interface
{
	// main and mainWindow can access protected member functions
	friend int main ( int, char *[] );
	friend class MainWindow;	// must call private functions for registration 

	public:
		Application ( const char * );
		virtual ~Application();

		// allow manipulation of top-level windows
		void manage();
		void unmanage();
		void iconify();

		// access functions
		Display *display() { return ( _display ); }
		XtAppContext appContext() { return ( _appContext ); }
		const char *applicationClass() { return ( _applicationClass ); }
		virtual const char *const className() { return ( "Application" ); }

	protected:
		// Xt interface functions
		virtual void initialize ( int *, char *[] );
		virtual void handleEvents();
		// support commonly-used data structures for convenience
		Display *_display;
		XtAppContext _appContext;
		char *_applicationClass;	// class name of application
		List<MainWindow*> _windows;
				
	private:
		void registerWindow ( MainWindow * );
		void unregisterWindow ( MainWindow * );
};

// pointer to single global instance of application
extern Application *application;

#endif
